<?php

namespace Besteron\Payment\Block;

use Magento\Framework\View\Element\Template;
use Magento\Framework\Registry;
use Magento\Checkout\Model\Session;
use Magento\Sales\Model\Order;
use Magento\Framework\HTTP\Client\Curl;
use Magento\Framework\Json\Helper\Data as JsonHelper;
use Magento\Framework\App\Config\ScopeConfigInterface;

class Redirect extends Template
{
    
    /**
     * Registry manager
     *
     * @var Registry
     */
    protected $_registry;
    
    /**
     * Order class
     *
     * @var Order
     */
    protected $_orderManager;
    
    /**
     * Core config reader
     *
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;
    
    /**
     * CURL class
     *
     * @var Curl
     */
     protected $curl;
     
    /**
     * JSON class for decoding
     *
     * @var Json\Helper\Data
     */
     protected $jsonHelper;

    /**
     * Init variables
     *
     * @param Template\Context $context
     * @param Registry $registry
     * @param Order $orderManager
     * @param ScopeConfigInterface $scopeConfig
     * @param Curl $curl
     * @param Json\Helper\Data $jsonHelper
     * @param array $data
     * @return $this
     */
    public function __construct(
        Template\Context $context,
        Registry $registry,
        Order $orderManager,
        ScopeConfigInterface $scopeConfig,
        Curl $curl,
        JsonHelper $jsonHelper,
        array $data = []
    ) {
        $this->_registry = $registry;
        $this->_orderManager = $orderManager;
        $this->scopeConfig = $scopeConfig;
        $this->curl = $curl;
        $this->jsonHelper = $jsonHelper;
        parent::__construct($context, $data);
    }

    /**
     * Get GoPay gateway URL
     *
     * @param string $suboption
     * @return string
     */
    public function getGatewayUrl($suboption = '')
    {
        // Load order
        $orderId = $this->_registry->registry('order_id');
        $order = $this->_orderManager->load($orderId);
		$orderCurrency = $order->getOrderCurrencyCode();
		
		// Address
		$address = $order->getShippingAddress();
		if(!$address) {
			$address = $order->getBillingAddress();
        }

        /* Items */
        $orderItems = [];
		
		/* Items Tax */
		$itemsTax = 0.00;
		
        /* Items */
        $items = $order->getAllItems();
        foreach ($items as $item) {
            if ((float) $item->getOriginalPrice() > 0) {
                $qty = (int) $item->getQtyOrdered();
                $amount = (($item->getRowTotalInclTax() / 1) * 100);
                $itemsTax = (float) $item->getTaxPercent();
                $orderItems[] = [
                    "name" => mb_substr($item->getName(), 0, 128),
                    "type" => 'ITEM',
                    "amount" => ($amount / $qty),
                    "count" => $qty,
					"vatRate" => $itemsTax
                ];
            }
        }

        /* Shipment */
        if ((float) $order->getShippingInclTax() != 0) {
            $orderItems[] = [
                "name" => mb_substr($order->getShippingDescription(), 0, 128),
                "type" => 'DELIVERY',
                "amount" => (int) bcmul($order->getShippingInclTax(), 100),
                "count" => 1,
				"vatRate" => $itemsTax
            ];
        }
        
        /* Discount */
        if ((float) $order->getDiscountAmount() != 0) {
            $orderItems[] = [
                "name" => 'Sleva',
                "type" => 'COUPON',
                "amount" => (int) bcmul($order->getDiscountAmount(), 100),
                "count" => 1,
				"vatRate" => $itemsTax
            ];
        }
		
		// Config params
        $cid = "" . $this->getConfigValue('payment/besteron/cid');
		$cidEur = "" . $this->getConfigValue('payment/besteron/cid_eur');
        $apiKey = "" . $this->getConfigValue('payment/besteron/api_key');
        $apiKeyEur = "" . $this->getConfigValue('payment/besteron/api_key_eur');
        $paymentKey = "" . $this->getConfigValue('payment/besteron/payment_key');
        $paymentKeyEur = "" . $this->getConfigValue('payment/besteron/payment_key_eur');
        $gatewayUrl = "" . $this->getConfigValue('payment/besteron/gateway_url');
        $displayInline = "" . $this->getConfigValue('payment/besteron/display_inline');
        $language = "" . $this->getConfigValue('payment/besteron/language');
        $paymentMethods = "" . $this->getConfigValue('payment/besteron/payment_methods');
        $validityTime = "" . $this->getConfigValue('payment/besteron/validity_time');
				
        // Get token
		if($orderCurrency == 'EUR') {
			$tokenData = [
				'grant_type' => 'client_credentials',
				'client_id' => $cidEur,
				'client_secret' => $paymentKeyEur
			];
		} else {
			$tokenData = [
				'grant_type' => 'client_credentials',
				'client_id' => $cid,
				'client_secret' => $paymentKey
			];
		}
                            
        $tokenResult = $this->curlReqeust(
            $gatewayUrl . "oauth2/token",
            'POST',
            ['Accept' => 'application/json', 'Content-Type' => 'application/x-www-form-urlencoded'],
            $tokenData
        );
        $accessToken = $tokenResult['access_token'];
		
		// Add comment
        $order->addStatusToHistory($order->getStatus(), 'Besteron - Token: ' . $accessToken);
        $order->save();
		
		// Create Payment
		$data['totalAmount'] = (int) bcmul($order->getGrandTotal(), 100);
        $data['currencyCode'] = $order->getOrderCurrencyCode();
        $data['orderNumber'] = $order->getIncrementId();
        $data['language'] = ((trim($language) == '') ? 'CS' : $language);
        $data['validityTime'] = (((int) $validityTime == 0) ? 3600 : (int) $validityTime);
		
		if($suboption != '') {
			$data['paymentMethods'] = array($suboption);
		} else {
			$data['paymentMethods'] = $this->getAllowedPaymentMethods($orderCurrency);
		}
			
		// Add APPLE and GOOGLE PAY
		if(in_array('CARD', $data['paymentMethods'])) {
			$data['paymentMethods'][] = 'APPLEPAY';
			$data['paymentMethods'][] = 'GOOGLEPAY';
		}
		
		$data['description'] = __('Objednávka') . ': ' . $order->getIncrementId();
        $data['buyer'] = [
            'email' => strtolower($address->getEmail()),
            'firstName' => mb_substr($address->getFirstname(), 0, 128),
            'lastName' => mb_substr($address->getLastname(), 0, 128),
			'delivery' => array(
				'streetLines' => array(
					mb_substr(implode(' ', $address->getStreet()), 0, 128)
				),
				'postalCode' => mb_substr($address->getPostcode(), 0, 16),
				'city' => mb_substr($address->getCity(), 0, 64),
				'countryCode' => $address->getCountryId()
			)
        ];
		
        $data['items'] = $orderItems;
        $data['callback'] = [
            'returnUrl' => $this->getUrl('besteron/gateway/return'),
            'notificationUrl' => $this->getUrl('besteron/gateway/notification')
        ];

        // Get GW URL
        $gatewayResult = $this->curlReqeust(
            $gatewayUrl . 'payment-intent',
            'POST',
            [
                'Accept' => 'application/json',
                'Content-Type' => 'application/json',
                'Authorization' => 'Bearer ' . $accessToken
			],
            json_encode($data)
        );
		
		if(array_key_exists('redirectUrl', $gatewayResult)) {
			// Add comment
			$order->setTransactionId($gatewayResult['transactionId']);
			$order->addStatusToHistory($order->getStatus(), 'Besteron - Transaction ID: ' . $gatewayResult['transactionId']);
			$order->save();
			
			return $gatewayResult['redirectUrl'];
		} else {
			$order->addStatusToHistory($order->getStatus(), 'Besteron Full Request - Error: ' . $gatewayResult['title']);
			$order->save();
			
			// Send limited request
			
			// Request data
			$data = array();			
			$data['totalAmount'] = (int) bcmul($order->getGrandTotal(), 100);
			$data['currencyCode'] = $order->getOrderCurrencyCode();
			$data['orderNumber'] = $order->getIncrementId();
			$data['language'] = ((trim($language) == '') ? 'CS' : $language);
			$data['validityTime'] = (((int) $validityTime == 0) ? 3600 : (int) $validityTime);
			
			if($suboption != '') {
				$data['paymentMethods'] = array($suboption);
			} else {
				$data['paymentMethods'] = $this->getAllowedPaymentMethods($orderCurrency);
			}
			
			// Add APPLE and GOOGLE PAY
			if(in_array('CARD', $data['paymentMethods'])) {
				$data['paymentMethods'][] = 'APPLEPAY';
				$data['paymentMethods'][] = 'GOOGLEPAY';
			}
			
			$data['items'] = [
				[
					"name" => 'Úhrada objednávky',
					"type" => 'ITEM',
					"amount" => (int) bcmul($order->getGrandTotal(), 100),
					"count" => 1
				]
            ];
			
			$data['callback'] = [
				'returnUrl' => $this->getUrl('besteron/gateway/return'),
				'notificationUrl' => $this->getUrl('besteron/gateway/notification')
			];
			
			$data['buyer'] = [
				'email' => strtolower($address->getEmail())
			];

			// Get GW URL
			$gatewayResultLimited = $this->curlReqeust(
				$gatewayUrl . 'payment-intent',
				'POST',
				[
					'Accept' => 'application/json',
					'Content-Type' => 'application/json',
					'Authorization' => 'Bearer ' . $accessToken
				],
				json_encode($data)
			);
			
			if(array_key_exists('redirectUrl', $gatewayResultLimited)) {
				// Add comment
				$order->setTransactionId($gatewayResultLimited['transactionId']);
				$order->addStatusToHistory($order->getStatus(), 'Besteron - Transaction ID: ' . $gatewayResultLimited['transactionId']);
				$order->save();
				
				return $gatewayResultLimited['redirectUrl'];
			} else {
				// Add comment
				$order->addStatusToHistory($order->getStatus(), 'Besteron Limited Request - Error: ' . $gatewayResultLimited['title']);
				$order->save();
				return '';
			}
		}		
    }
    
    /**
     * Get config value from  core_config_data table
     *
     * @param string $path
     * @return string
     */
    private function getConfigValue($path)
    {
        return $this->scopeConfig->getValue($path, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }
	
    /**
     * Get allowed payment methods by order currency
     *
     * @param string $currency
     * @return array
     */
    private function getAllowedPaymentMethods($currency)
    {
		$allowedMethods = array();
		$configAllowedMethods = explode(',', "" . $this->getConfigValue('payment/besteron/payment_methods'));
		$paymentOptions = array(
			'AGBACZPP' => array('title' => 'MONETA MONEY BANK, A.S.', 'allowed_currencies' => array('CZK')),
			'BACXCZPP' => array('title' => 'UNICREDIT BANK CZECH REPUBLIC AND SLOVAKIA, A.S.', 'allowed_currencies' => array('CZK')),
			'BREXCZPP' => array('title' => 'MBANK S.A.', 'allowed_currencies' => array('CZK')),
			'FIOBCZPP' => array('title' => 'FIO BANKA, A.S.', 'allowed_currencies' => array('CZK')),
			'GIBACZPX' => array('title' => 'CESKA SPORITELNA A.S.', 'allowed_currencies' => array('CZK')),
			'KOMBCZPP' => array('title' => 'KOMERCNI BANKA A.S.', 'allowed_currencies' => array('CZK')),
			'RZBCCZPP' => array('title' => 'RAIFFEISENBANK A.S.', 'allowed_currencies' => array('CZK')),
			'GIBASKBX' => array('title' => 'SLOVENSKA SPORITELNA, A.S.', 'allowed_currencies' => array('EUR')),
			'POBNSKBA' => array('title' => 'POSTOVA BANKA A.S.', 'allowed_currencies' => array('EUR')),
			'SUBASKBX' => array('title' => 'VSEOBECNA UVEROVA BANKA A.S.', 'allowed_currencies' => array('EUR')),
			'TATRSKBX' => array('title' => 'TATRA BANKA A.S.', 'allowed_currencies' => array('EUR')),
			'UNCRSKBX' => array('title' => 'UNICREDIT BANK CZ AND SK, SK BRANCH', 'allowed_currencies' => array('EUR')),
			'VIAMO' => array('title' => 'MONETA MONEY BANK, A.S.', 'allowed_currencies' => array('EUR')),
			'CARD' => array('title' => 'CREDIT CARD', 'allowed_currencies' => array('CZK','EUR')),
			'VIAMO' => array('title' => 'VIAMO A.S.', 'allowed_currencies' => array('EUR')),
			'APPLEPAY' => array('title' => 'APPLE PAY', 'allowed_currencies' => array('EUR')),
			'GOOGLEPAY' => array('title' => 'GOOGLE PAY', 'allowed_currencies' => array('EUR'))
		);
		
		for($i = 0; $i < count($configAllowedMethods); $i++) {
			$methodInfo = $paymentOptions[$configAllowedMethods[$i]];
			if(in_array($currency, $methodInfo['allowed_currencies'])) {
				$allowedMethods[] = $configAllowedMethods[$i];
			}
		}
		
        return $allowedMethods;
    }
    
    /**
     * Make a CURL request
     *
     * @param string $url
     * @param string $method
     * @param array $headers
     * @param array $data
     * @param string $httpUser
     * @param string $httpPassword
     * @return array
     */
    private function curlReqeust($url, $method, $headers = null, $data = null, $httpUser = '', $httpPassword = '')
    {
        $this->curl->setOption(CURLOPT_TIMEOUT, 400);
        $this->curl->setOption(CURLOPT_RETURNTRANSFER, true);
        $this->curl->setOption(CURLOPT_CONNECTTIMEOUT, 0);
        $this->curl->setOption(CURLOPT_HEADER, 0);
        $this->curl->setOption(CURLOPT_CUSTOMREQUEST, $method);
        $this->curl->setOption(CURLOPT_SSL_VERIFYPEER, false);
        
        foreach ($headers as $headerName => $headerValue) {
            $this->curl->addHeader($headerName, $headerValue);
        }
        
        if ($httpUser != '' && $httpPassword != '') {
            $this->curl->setOption(CURLOPT_USERPWD, $httpUser . ":" . $httpPassword);
        }
        
        if ($method == 'POST') {
            $this->curl->post($url, $data);
        } else {
            $this->curl->get($url);
        }
        
        $response = $this->curl->getBody();
        return $this->jsonHelper->jsonDecode($response);
    }
}
