<?php

namespace Besteron\Payment\Controller\Gateway;

use Magento\Framework\View\Element\Template;
use Magento\Framework\Registry;
use Magento\Checkout\Model\Session;
use Magento\Sales\Model\Order;
use Magento\Framework\HTTP\Client\Curl;
use Magento\Framework\Json\Helper\Data as JsonHelper;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\ResponseFactory;
use Magento\Framework\App\ResourceConnection;

class Notification extends \Magento\Framework\App\Action\Action {
    
    /**
     * Registry manager
     *
     * @var Registry
     */
    protected $_registry;
    
    /**
     * Order class
     *
     * @var Order
     */
    protected $_orderManager;
    
    /**
     * Core config reader
     *
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;
    
    /**
     * Page result
     *
     * @var PageFactory
     */
    protected $resultFactory;
    
    /**
     * Response output
     *
     * @var ResponseFactory
     */
     protected $responseFactory;
    
    /**
     * CURL class
     *
     * @var Curl
     */
     protected $curl;
     
    /**
     * JSON class for decoding
     *
     * @var Json\Helper\Data
     */
     protected $jsonHelper;
     
    /**
     * DB resource class
     *
     * @var Resource
     */
     protected $res;
     
    /**
     * DB connection class
     *
     * @var Connection
     */
     protected $db;	 
	 

    /**
     * Init variables
     *
     * @param Context $context
     * @param Registry $registry
     * @param Order $orderManager
     * @param ScopeConfigInterface $scopeConfig
     * @param ResultFactory $resultFactory
     * @param ResponseFactory $responseFactory
     * @param Curl $curl
     * @param Json\Helper\Data $jsonHelper
	 * @param ResourceConnection $res
     * @param array $data
     * @return $this
     */
    public function __construct(
        Context $context,
        Registry $registry,
        Order $orderManager,
        ScopeConfigInterface $scopeConfig,
        ResultFactory $resultFactory,
		ResponseFactory $responseFactory,
        Curl $curl,
        JsonHelper $jsonHelper,
		ResourceConnection $res,
        array $data = []
    ) {
        $this->_registry = $registry;
        $this->_orderManager = $orderManager;
        $this->scopeConfig = $scopeConfig;
        $this->resultFactory = $resultFactory;
		$this->responseFactory = $responseFactory;
        $this->curl = $curl;
        $this->jsonHelper = $jsonHelper;
		$this->res = $res;
		$this->db = $res->getConnection();
        parent::__construct($context);
    }

    /**
     * Execute view action
     * 
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute() {
		
        $transactionId = $this->getRequest()->getParam('transactionId');

        if (trim($transactionId) != '') {
			
			// Find order by transaction ID 
			$orderExist = $this->db->fetchAll("SELECT * FROM " . $this->res->getTableName('sales_order') . " WHERE transaction_id = '" . addslashes($transactionId) . "'");
			$orderCurrency = $orderExist[0]['order_currency_code'];
			
			// Config params
			$cid = "" . $this->getConfigValue('payment/besteron/cid');
			$cidEur = "" . $this->getConfigValue('payment/besteron/cid_eur');
			$apiKey = "" . $this->getConfigValue('payment/besteron/api_key');
			$apiKeyEur = "" . $this->getConfigValue('payment/besteron/api_key_eur');
			$paymentKey = "" . $this->getConfigValue('payment/besteron/payment_key');
			$paymentKeyEur = "" . $this->getConfigValue('payment/besteron/payment_key_eur');
			$passiveGatewayUrl = "" . $this->getConfigValue('payment/besteron/passive_gateway_url');
			$displayInline = "" . $this->getConfigValue('payment/besteron/display_inline');
			$language = "" . $this->getConfigValue('payment/besteron/language');
			$paymentMethods = "" . $this->getConfigValue('payment/besteron/payment_methods');
			$validityTime = "" . $this->getConfigValue('payment/besteron/validity_time');
				
			// Get token
			if($orderCurrency == 'EUR') {
				$tokenData = [
					'grant_type' => 'client_credentials',
					'client_id' => $cidEur,
					'client_secret' => $apiKeyEur
				];
			} else {
				$tokenData = [
					'grant_type' => 'client_credentials',
					'client_id' => $cid,
					'client_secret' => $apiKey
				];
			}
								
			$tokenResult = $this->curlReqeust(
				$passiveGatewayUrl . 'oauth2/token',
				'POST',
				['Accept' => 'application/json', 'Content-Type' => 'application/x-www-form-urlencoded'],
				$tokenData
			);
			$accessToken = $tokenResult['access_token'];
			
            // Get Payment Info
			$paymentInfo = $this->curlReqeust(
				'https://passive.besteron.com/api/payment-intents/' . $transactionId,
				'POST',
				[
					'Accept' => 'application/json',
					'Content-Type' => 'application/json',
					'Authorization' => 'Bearer ' . $accessToken
				]
			);
			
			if(array_key_exists('transaction', $paymentInfo)) {
				
				// Payment status
				$paymentStatus = $paymentInfo['transaction']['status'];
				$incrementId = $paymentInfo['transaction']['paymentIntent']['orderNumber'];
				
				if ($paymentStatus == 'Completed') {

					// Payment complete
					$order = $this->_orderManager->loadByIncrementId($incrementId);
					$order->setState(\Magento\Sales\Model\Order::STATE_PROCESSING, true);
					$order->setStatus(\Magento\Sales\Model\Order::STATE_PROCESSING);
					$order->addStatusToHistory($order->getStatus(), 'Besteron - Status - ' . $paymentStatus);
					$order->save();
					
				} else if ($paymentStatus == 'Created' || $paymentStatus == 'WaitingForConfirmation' || $paymentStatus == 'ManualAttentionRequired') {
					
					// Check if order status still waiting for payment
					if($order->getStatus() == \Magento\Sales\Model\Order::STATE_PENDING_PAYMENT) {
						// Waiting for payment
						$order = $this->_orderManager->loadByIncrementId($incrementId);
						$order->setState(\Magento\Sales\Model\Order::STATE_PROCESSING, true);
						$order->setStatus(\Magento\Sales\Model\Order::STATE_PENDING_PAYMENT);
						$order->addStatusToHistory($order->getStatus(), 'Besteron - Status - ' . $paymentStatus);
						$order->save();
					}
					
				} else {					
					// Check if order status still waiting for payment
					if($order->getStatus() == \Magento\Sales\Model\Order::STATE_PENDING_PAYMENT) {
						// Cancel order
						$order = $this->_orderManager->loadByIncrementId($incrementId);
						$order->addStatusToHistory($order->getStatus(), 'Besteron - Status - ' . $paymentStatus);
						$order->setState(\Magento\Sales\Model\Order::STATE_CANCELED, true);
						$order->setStatus(\Magento\Sales\Model\Order::STATE_CANCELED);
						$order->save();
						
						// Set quote to cart
						if(!$this->checkoutSession->hasQuote()) {
							try {
								$this->checkoutSession->restoreQuote();		
							} catch (\Exception $e) {
								echo $e->getMessage();
								exit;
							}
						}
					}
				}
			}
        }

    }
    
    /**
     * Get config value from  core_config_data table
     *
     * @param string $path
     * @return string
     */
    private function getConfigValue($path)
    {
        return $this->scopeConfig->getValue($path, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }
    
    /**
     * Make a CURL request
     *
     * @param string $url
     * @param string $method
     * @param array $headers
     * @param array $data
     * @param string $httpUser
     * @param string $httpPassword
     * @return array
     */
    private function curlReqeust($url, $method, $headers = null, $data = null, $httpUser = '', $httpPassword = '')
    {
        $this->curl->setOption(CURLOPT_TIMEOUT, 400);
        $this->curl->setOption(CURLOPT_RETURNTRANSFER, true);
        $this->curl->setOption(CURLOPT_CONNECTTIMEOUT, 0);
        $this->curl->setOption(CURLOPT_HEADER, 0);
        $this->curl->setOption(CURLOPT_CUSTOMREQUEST, $method);
        $this->curl->setOption(CURLOPT_SSL_VERIFYPEER, false);
        
        foreach ($headers as $headerName => $headerValue) {
            $this->curl->addHeader($headerName, $headerValue);
        }
        
        if ($httpUser != '' && $httpPassword != '') {
            $this->curl->setOption(CURLOPT_USERPWD, $httpUser . ":" . $httpPassword);
        }
        
        if ($method == 'POST') {
            $this->curl->post($url, $data);
        } else {
            $this->curl->get($url);
        }
        
        $response = $this->curl->getBody();
		
        return $this->jsonHelper->jsonDecode($response);
    }

}
