<?php

namespace Besteron\Payment\Controller\Gateway;

use Magento\Framework\Registry;
use Magento\Framework\App\Action\Context;
use Magento\Framework\View\Result\PageFactory;
use Magento\Framework\Controller\ResultFactory;
use Magento\Checkout\Model\Session;
use Magento\Quote\Model\QuoteFactory;
use Magento\Checkout\Model\Cart;
use Besteron\Payment\Block\Redirect as RedirectBlock;

class Redirect extends \Magento\Framework\App\Action\Action
{

    /**
     * Page response
     *
     * @var PageFactory
     */
    protected $resultPageFactory;
    
    /**
     * Page result
     *
     * @var PageFactory
     */
    protected $resultFactory;
        
    /**
     * Checkout sesssion class
     *
     * @var Session
     */
    protected $checkoutSession;
        
    /**
     * Quote class
     *
     * @var QuoteFactory
     */
    protected $quoteFactory;
    
    /**
     * Registry manager
     *
     * @var Registry
     */
    protected $registry;
    
    /**
     * Besteron redirect class
     *
     * @var RedirectBlock
     */
    protected $redirectBlock;
    
    /**
     * Cart management class
     *
     * @var Cart
     */
    protected $cart;

    /**
     * Constructor
     *
     * @param Context $context
     * @param PageFactory $resultPageFactory
     * @param ResultFactory $resultFactory
     * @param Session $checkoutSession
     * @param QuoteFactory $quoteFactory
     * @param Cart $cart
     * @param RedirectBlock $redirectBlock
     * @param Registry $registry
     * @return $this
     */
    public function __construct(
        Context $context,
        PageFactory $resultPageFactory,
        ResultFactory $resultFactory,
        Session $checkoutSession,
        QuoteFactory $quoteFactory,
        Cart $cart,
        RedirectBlock $redirectBlock,
        Registry $registry
    ) {
        $this->resultPageFactory = $resultPageFactory;
        $this->checkoutSession = $checkoutSession;
        $this->registry = $registry;
        $this->quoteFactory = $quoteFactory;
        $this->cart = $cart;
        $this->resultFactory = $resultFactory;
        $this->redirectBlock = $redirectBlock;
        parent::__construct($context);
    }

    /**
     * Execute view action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $order = $this->checkoutSession->getLastRealOrder();
        $orderId = $order->getEntityId();
		$suboption = "" . $this->getRequest()->getParam('suboption');
		
        // Set order ID into registry
        $this->registry->register('order_id', $orderId);
        
        // Generate URL
        $url = $this->redirectBlock->getGatewayUrl($suboption);
		
		if($url == '') {
			return $this->resultPageFactory->create();
		} else {
			// Set redirect
			$redirect = $this->resultFactory->create(\Magento\Framework\Controller\ResultFactory::TYPE_REDIRECT);
			$redirect->setUrl($url);
		}

        // Redirect
        return $redirect;
    }
}
