<?php

namespace Besteron\Payment\Model;

use Magento\Checkout\Model\ConfigProviderInterface;
use Magento\Framework\Escaper;
use Magento\Payment\Helper\Data as PaymentHelper;
use Magento\Checkout\Model\Session;
use Magento\Framework\App\Config\ScopeConfigInterface;

class MethodConfigProvider implements ConfigProviderInterface
{
    /**
     * @var string[]
     */
    protected $methodCodes = [
        'besteron'
    ];

    /**
     * @var \Magento\Payment\Model\Method\AbstractMethod[]
     */
    protected $methods = [];

    /**
     * @var Escaper
     */
    protected $escaper;
    protected $scopeConfig;
    protected $checkoutSession;

    /**
     * Init all payment methods
     *
     * @param PaymentHelper $paymentHelper
     * @param Escaper $escaper
     */
    public function __construct(
        PaymentHelper $paymentHelper,
        Escaper $escaper,
		Session $checkoutSession,
		ScopeConfigInterface $scopeConfig
    ) {
        $this->escaper = $escaper;
        $this->scopeConfig = $scopeConfig;
		$this->checkoutSession = $checkoutSession;
		
        foreach ($this->methodCodes as $code) {
            $this->methods[$code] = $paymentHelper->getMethodInstance($code);
        }
    }
    
    /**
     * Load payment instructions for all methods
     *
     * @return array
     */
    public function getConfig()
    {
        $config = [];
		$quoteCurrency = $this->checkoutSession->getQuote()->getQuoteCurrencyCode();
		$allowedMethods = explode(',', "" . $this->getConfigValue('payment/besteron/payment_methods'));
		$displayType = "" . $this->getConfigValue('payment/besteron/display_type');
		$methodsOutput = '';
		$paymentOptions = array(
			'AGBACZPP' => array('title' => 'MONETA MONEY BANK, A.S.', 'allowed_currencies' => array('CZK')),
			'BACXCZPP' => array('title' => 'UNICREDIT BANK CZECH REPUBLIC AND SLOVAKIA, A.S.', 'allowed_currencies' => array('CZK')),
			'BREXCZPP' => array('title' => 'MBANK S.A.', 'allowed_currencies' => array('CZK')),
			'FIOBCZPP' => array('title' => 'FIO BANKA, A.S.', 'allowed_currencies' => array('CZK')),
			'GIBACZPX' => array('title' => 'CESKA SPORITELNA A.S.', 'allowed_currencies' => array('CZK')),
			'KOMBCZPP' => array('title' => 'KOMERCNI BANKA A.S.', 'allowed_currencies' => array('CZK')),
			'RZBCCZPP' => array('title' => 'RAIFFEISENBANK A.S.', 'allowed_currencies' => array('CZK')),
			'GIBASKBX' => array('title' => 'SLOVENSKA SPORITELNA, A.S.', 'allowed_currencies' => array('EUR')),
			'POBNSKBA' => array('title' => 'POSTOVA BANKA A.S.', 'allowed_currencies' => array('EUR')),
			'SUBASKBX' => array('title' => 'VSEOBECNA UVEROVA BANKA A.S.', 'allowed_currencies' => array('EUR')),
			'TATRSKBX' => array('title' => 'TATRA BANKA A.S.', 'allowed_currencies' => array('EUR')),
			'UNCRSKBX' => array('title' => 'UNICREDIT BANK CZ AND SK, SK BRANCH', 'allowed_currencies' => array('EUR')),
			'VIAMO' => array('title' => 'MONETA MONEY BANK, A.S.', 'allowed_currencies' => array('EUR')),
			'CARD' => array('title' => 'CREDIT CARD', 'allowed_currencies' => array('CZK','EUR')),
			'VIAMO' => array('title' => 'VIAMO A.S.', 'allowed_currencies' => array('EUR')),
			'APPLEPAY' => array('title' => 'APPLE PAY', 'allowed_currencies' => array('EUR')),
			'GOOGLEPAY' => array('title' => 'GOOGLE PAY', 'allowed_currencies' => array('EUR'))
		);
		
		for($i = 0; $i < count($allowedMethods); $i++) {
			$methodInfo = $paymentOptions[$allowedMethods[$i]];
			
			if(in_array($quoteCurrency, $methodInfo['allowed_currencies'])) {
				$methodsOutput .= '<li><input type="radio" class="besteron-payment-option" name="besteron_payment_option" id="' . $allowedMethods[$i] . '" value="' . $allowedMethods[$i] . '" /><label for="' . $allowedMethods[$i] . '" class="' . $allowedMethods[$i] . '">' . $methodInfo['title'] . '</label></li>';
			}
		}
		        
        foreach ($this->methodCodes as $code) {
            $config['payment']['instructions'][$code] = 
				$this->methods[$code]->getInstructions() .				
				(($displayType == 'advance') ?
				'<div class="besteron-payment-options">' .
					'<ul>' .
						$methodsOutput .
					'</ul>' .
				'</div>'				
				: '')
			;
        }
        
        return $config;
    }
    
    /**
     * Get config value from  core_config_data table
     *
     * @param string $path
     * @return string
     */
    private function getConfigValue($path)
    {
        return $this->scopeConfig->getValue($path, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }
}
